package inf101.v18.connectfour;

import inf101.v18.connectfour.interfaces.*;

import javax.swing.*;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * This is a "base" class, which lets the user choose a game.
 */
public class Main {
    private static final int BOARD_WIDTH = 7;
    private static final int BOARD_HEIGHT = 6;

    public static void main(String[] args) {
        ArrayList<String> gamemodes = new ArrayList<>();
        gamemodes.add("Connect Four");
        gamemodes.add("Connect Four classic");
        gamemodes.add("Tic-Tac-Toe");
        if (System.console() != null) {
            gamemodes.add("Connect Four CLI");
            gamemodes.add("Tic-Tac-Toe CLI");
        }

        JOptionPane.showMessageDialog(
                null,
                "This software contains several games:\n" +
                        "* Connect Four is a custom version with customizable number of players.\n" +
                        "* Connect Four classic is a version closer to that of the original analogue version. Two human players are required.\n" +
                        "* Tic-Tac-Toe is a game built upon the same framework as Connect Four, but behaves quite differently. Two human players are required.\n" +
                        "* Connect Four CLI is a CLI implementation of connect four. It supports two players or one player and one ai.\n" +
                        "* Tic-Tac-Toe CLI is a CLI implementation of Tic-Tac-Toe. It supports two players or one player and one ai.",
                "Game-types",
                JOptionPane.INFORMATION_MESSAGE);
        String selectedValue = (String)JOptionPane.showInputDialog(
                null,
                "Select a game from the list below:",
                "Game type",
                JOptionPane.QUESTION_MESSAGE,
                null,
                gamemodes.toArray(),
                gamemodes.get(0));


        if (selectedValue == null) {
            System.exit(0);
        }
        boolean playAgain = true;
        switch (selectedValue) {
            case "Tic-Tac-Toe":
                new TicTacToeGUI();
                break;
            case "Connect Four classic":
                new ConnectFourClassicGUI(BOARD_WIDTH, BOARD_HEIGHT);
                break;
            case "Connect Four":
                new ConnectFourGUI(BOARD_WIDTH, BOARD_HEIGHT);
                break;
            case "Connect Four CLI":
                while (playAgain) {
                    playAgain = CLILoop(new ConnectFourCLI(7, 6));
                }
                break;
            case "Tic-Tac-Toe CLI":
                while (playAgain) {
                    playAgain = CLILoop(new TicTacToeCLI());
                }
                break;
             default:
                System.exit(0);
        }
    }

    private static boolean CLILoop(IGameInterface gameInterface) {
        Scanner in = new Scanner(System.in);
        while (!gameInterface.hasWon() && !gameInterface.hasDrawn()) {
            gameInterface.displayBoard();
            gameInterface.doTurn();
        }
        if (gameInterface.hasDrawn() && !gameInterface.hasWon()) {
            gameInterface.displayMessage("It's a draw.");
        }
        gameInterface.displayMessage("Do you want to play again? (y/n)");
        return in.nextLine().toLowerCase().contains("y");
    }
}
