package inf101.v18.connectfour.board;

import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.player.IPlayer;

import java.util.List;

/**
 * An interface representing a game board.
 */
public interface IBoard extends IGrid<IGameObject> {
    /**
     * Adds a new player to the board.
     *
     * @param p The player to add
     * @return  True if the player is valid
     */
    boolean addPlayer(IPlayer p);

    /**
     * Gets the next player from the board.
     *
     * @return  A player
     */
    IPlayer nextPlayer();

    /**
     * Find out if someone has won.
     *
     * @return True if someone won
     */
    boolean won();

    /**
     * Flushes all values in the board.
     */
    void flush();

    /**
     * Gets a copy of the players list.
     *
     * @return  A list of players
     */
    List<IPlayer> getPlayers();

    /**
     * Makes a copy of the entire board.
     *
     * @return  A new, but identical board
     */
    IBoard copyAll();
}
