package inf101.v18.connectfour.board;

import inf101.v18.connectfour.objects.IGameObject;

/**
 * An IBoard with added methods for vertical interactions.
 */
public interface IVerticalBoard extends IBoard {
    /**
     * Retrieves the position of the "topmost" element in a column.
     *
     * Will return -1 if the column is empty.
     *
     * @param column    The target column
     * @return          The y-position of the top element
     */
    int getTopObject(int column);

    /**
     * Retrieves the position of the "topmost" element of class clazz in a column.
     *
     * Will return -1 if the column is empty.
     *
     * @param column    The target column
     * @param clazz     The target class
     * @return          The y-position of the top element
     */
    int getTopObject(int column, Class<?> clazz);

    /**
     * Drops an element into column i.
     *
     * i must be greater than or equal to 0 and less than getWidth.
     *
     * @param i         The target column
     * @param element   The element to be dropped
     * @return          True if the column had space for the element
     */
    boolean drop(int i, IGameObject element);

    /**
     * Moves all floating elements one tile down.
     */
    void gravitate();

    /**
     * Tries to count the amount of steps required to gravitate all floating objects.
     *
     * @return  An integer
     */
    int floatingObjects();

    @Override
    IVerticalBoard copyAll();
}
