package inf101.v18.connectfour.board;

import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.IPlayerObject;

import java.util.ArrayList;
import java.util.List;

/**
 * A board which supports dropping objects and gravity.
 */
public class VerticalBoard extends Board implements IVerticalBoard {
    public VerticalBoard(int width, int height, IGameObject empty, List<IWinCondition> conditions) {
        super(width, height, empty, conditions);
    }

    /**
     * Checks if anyone has won.
     *
     * @param i The column of the last dropped object
     */
    private void checkWin(int i) {
        for (IWinCondition con : conditionList) {
            if (con.hasWon(i, getTopObject(i, IPlayerObject.class), this)) {
                won = true;
            }
        }
    }

    @Override
    public int floatingObjects() {
        int maxFloat = 0;
        for (int i = 0; i < getWidth(); i++) {
            int topPos = -1;
            int holes = 0;
            for (int j = getHeight() - 1; j > -1; j--) {
                if (!(empty.getClass().isInstance(grid.get(i, j)))) {
                    topPos = j;
                } else if (topPos != -1) {
                    holes++;
                }
            }
            if (holes > maxFloat) {
                maxFloat = holes;
            }
        }
        return maxFloat;
    }

    @Override
    public void gravitate() {
        for (int i = 0; i < getWidth(); i++) {
            for (int j = 1; j < getHeight(); j++) {
                if (!(empty.getClass().isInstance(grid.get(i, j))) && empty.getClass().isInstance(grid.get(i, j - 1))) {
                    grid.set(i, j - 1, grid.get(i, j));
                    grid.set(i, j, empty);
                }
            }
        }
    }

    @Override
    public boolean drop(int i, IGameObject element) {
        if (i >= getWidth() || i < 0) {
            throw new IllegalArgumentException("Index out of bounds.");
        }
        if (element == null) {
            throw new IllegalArgumentException("This grid does not accept null.");
        }
        for (int j = 0; j < getHeight(); j++) {
            if (empty.getClass().isInstance(get(i, j))) {
                set(i, j, element);
                if (element instanceof IPlayerObject) {
                    checkWin(i);
                }
                return true;
            }
        }
        return false;
    }

    @Override
    public int getTopObject(int i) {
        if (i < 0 || i >= getWidth()) {
            throw new IllegalArgumentException("Index out of bounds.");
        }
        for (int j = getHeight() - 1; j > -1; j--) {
            if (!(empty.getClass().isInstance(get(i, j)))) {
                return j;
            }
        }
        return -1;
    }

    @Override
    public int getTopObject(int i, Class<?> clazz) {
        if (i < 0 || i >= getWidth()) {
            throw new IllegalArgumentException("Index out of bounds.");
        }
        for (int j = getHeight() - 1; j > -1; j--) {
            if (clazz.isInstance(get(i, j))) {
                return j;
            }
        }
        return -1;
    }

    @Override
    public IVerticalBoard copyAll() {
        VerticalBoard newBoard = new VerticalBoard(getWidth(), getHeight(), empty, conditionList);
        newBoard.players = new ArrayList<>(players);
        newBoard.won = won;
        newBoard.grid = copy();
        newBoard.playersTurn = playersTurn;
        return newBoard;
    }
}
