package inf101.v18.connectfour.interfaces;

import inf101.v18.connectfour.board.IVerticalBoard;
import inf101.v18.connectfour.board.VerticalBoard;
import inf101.v18.connectfour.events.winning.FourInARow;
import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.objects.Empty;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.IPlayerObject;
import inf101.v18.connectfour.player.IPlayer;
import inf101.v18.connectfour.player.Player;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

/**
 * A GUI implementation of connect four.
 * This implementation tries to closely replicate the "original" version.
 */
public class ConnectFourClassicGUI extends ConnectGUI {
    public ConnectFourClassicGUI(int width, int height) {
        if (width < 3 || height < 3) {
            throw new IllegalArgumentException("Invalid game size.");
        }
        ArrayList<IWinCondition> con = new ArrayList<>();
        con.add(new FourInARow());
        this.board = new VerticalBoard(width, height, new Empty(), con);
        for (int i = 0; i < 2; i++) {
            String playerName = "";
            while (playerName.equals("")) {
                playerName = JOptionPane.showInputDialog(String.format("Name of player %d: ", i+1));
                if (playerName == null) {
                    System.exit(0);
                }
            }
            Color color;
            if (i == 0) {
                color = new Color(0, 200, 0);
            } else {
                color = new Color(200, 0, 0);
            }
            Player player = new Player(playerName, color);
            if (!this.board.addPlayer(player)) {
                JOptionPane.showMessageDialog(null, "Player is not unique. Please try again.", "Error", JOptionPane.ERROR_MESSAGE);
                i--;
            }
        }
        initialize(width, height);
    }

    /**
     * Initialized the gui.
     *
     * @param width     The width of the game grid
     * @param height    The height of the game grid
     */
    private void initialize(int width, int height) {
        JFrame frame = new JFrame("Four in a row");
        frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        int fullWidth = BUTTON_WIDTH * width + (WIDTH_GAP + 1) * width;
        int fullHeight = BUTTON_HEIGHT + (TILE_HEIGHT * height) + HEIGHT_GAP * (height) + TILE_HEIGHT;

        this.frame = frame;
        JPanel container = new JPanel();
        container.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        container.setPreferredSize(new Dimension(fullWidth, fullHeight + TILE_HEIGHT + BUTTON_HEIGHT));
        frame.add(container);

        JPanel panel3 = new JPanel();
        this.resetButton = new JButton("Reset");
        this.resetButton.addActionListener(this);
        panel3.add(this.resetButton);
        for (IPlayer player : board.getPlayers()) {
            JLabel playerLabel = new JLabel(player.getName());
            playerLabel.setForeground(player.getColor());
            panel3.add(playerLabel);
        }
        container.add(panel3);

        JPanel panel2 = new JPanel();
        this.mainPanel = panel2;
        panel2.setBackground(new Color(0, 0, 200));
        panel2.setLayout(new FlowLayout(FlowLayout.CENTER, HEIGHT_GAP, WIDTH_GAP));
        panel2.setPreferredSize(new Dimension(fullWidth, fullHeight));
        dropButtons = new JButton[width];
        initializeDropButtons(width, panel2);
        tiles = new JTextPane[width * height];
        readTiles();
        container.add(panel2);

        infoLabel = new JLabel();
        infoLabel.setPreferredSize(new Dimension(fullWidth, BUTTON_HEIGHT));
        infoLabel.setBorder(new EmptyBorder(10, 10, 10, 10));
        container.add(infoLabel);

        frame.validate();
        frame.pack();
        frame.setVisible(true);
        currentPlayer = board.nextPlayer();
        displayMessage("It's " + currentPlayer.getName() + "'s turn.");
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        reset(e);
        if (!(board.won() || board.isFull())) {
            for (int i = 0; i < dropButtons.length; i++) {
                if (e.getSource() == dropButtons[i]) {
                    if (dropObject(i)) {
                        displayBoard();
                        if (!board.won()) {
                            if (board.isFull()) {
                                displayMessage("It's a draw");
                            } else {
                                currentPlayer = board.nextPlayer();
                                displayMessage("It's " + currentPlayer.getName() + "'s turn.");
                            }
                        } else {
                            IGameObject winnerObject = board.get(i, ((IVerticalBoard)board).getTopObject(i, IPlayerObject.class));
                            displayMessage(((IPlayerObject)winnerObject).getOwner().getName() + " won.");
                        }
                    } else {
                        displayMessage("That column is full.");
                    }
                }
            }
        }
    }

    @Override
    public boolean placePlayerObject(int x, int y) throws IllegalAccessException {
        throw new IllegalAccessException("Method not allowed in a vertical board.");
    }
}
