package inf101.v18.connectfour.interfaces;

import inf101.v18.connectfour.board.IVerticalBoard;
import inf101.v18.connectfour.board.VerticalBoard;
import inf101.v18.connectfour.events.random.Block;
import inf101.v18.connectfour.events.random.Bomb;
import inf101.v18.connectfour.events.random.IRandomEvent;
import inf101.v18.connectfour.events.winning.FourInARow;
import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.objects.Empty;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.IPlayerObject;
import inf101.v18.connectfour.player.AI;
import inf101.v18.connectfour.player.IPlayer;
import inf101.v18.connectfour.player.Player;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

/**
 * A custom version of connect four.
 */
public class ConnectFourGUI extends ConnectGUI {
    private final IRandomEvent bomb = new Bomb();
    private final IRandomEvent block = new Block();
    private JLabel eventLabel;

    @Override
    public boolean placePlayerObject(int x, int y) {
        return false;
    }

    //A list of colors which can be used without conflict.
    private enum Colors {GREEN, RED, BLUE, ORANGE, YELLOW, PINK, BROWN, TEAL}

    public ConnectFourGUI(int width, int height) {
        if (width < 3 || height < 3 || height > 50 || width > 50) {
            throw new IllegalArgumentException("Invalid game size.");
        }
        ArrayList<IWinCondition> con = new ArrayList<>();
        con.add(new FourInARow());
        this.board = new VerticalBoard(width, height, new Empty(), con);
        int playerNum = 0;
        Colors[] colorValues = Colors.values();
        while (playerNum == 0) {
            try {
                String num = JOptionPane.showInputDialog("Number of players: ");
                if (num == null) {
                    System.exit(0);
                }
                playerNum = Integer.parseInt(num);
                if (playerNum > colorValues.length) {
                    JOptionPane.showMessageDialog(null, "Number of players can't be higher than " + colorValues.length, "Error", JOptionPane.ERROR_MESSAGE);
                    playerNum = 0;

                }
            } catch (NumberFormatException e) {
                JOptionPane.showMessageDialog(null, "Invalid input.", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
        int aiNum = -1;
        while (aiNum == -1) {
            try {
                String num = JOptionPane.showInputDialog("Number of ai: ");
                if (num == null) {
                    System.exit(0);
                }
                aiNum = Integer.parseInt(num);
                if (aiNum + playerNum > colorValues.length) {
                    JOptionPane.showMessageDialog(null, "Number of ai + players can't be higher than " + colorValues.length, "Error", JOptionPane.ERROR_MESSAGE);
                    aiNum = 0;

                }
            } catch (NumberFormatException e) {
                JOptionPane.showMessageDialog(null, "Invalid input.", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
        for (int i = 0; i < playerNum; i++) {
            String playerName = "";
            while (playerName.equals("")) {
                playerName = JOptionPane.showInputDialog(String.format("Name of player %d: ", i + 1));
                if (playerName == null) {
                    System.exit(0);
                }
            }
            Color color = colorsToColor(Colors.values()[i]);
            Player player = new Player(playerName, color);
            if (!this.board.addPlayer(player)) {
                JOptionPane.showMessageDialog(null, "Player is not unique. Please try again.", "Error", JOptionPane.ERROR_MESSAGE);
                i--;
            }
        }
        for (int i = 0; i < aiNum; i++) {
            if (!this.board.addPlayer(new AI("AI" + (i + 1), colorsToColor(colorValues[playerNum + i])))) {
                i--;
            }
        }
        initialize(width, height);
    }

    /**
     * Converts values in the Colors enum to actual colors.
     *
     * @param color An enum value
     * @return      An awt Color
     */
    private Color colorsToColor(Colors color) {
        switch (color) {
            case RED:
                return new Color(209,17,65);
            case GREEN:
                return new Color(0,177,89);
            case BLUE:
                return new Color(0, 174, 219);
            case ORANGE:
                return new Color(243, 119, 53);
            case YELLOW:
                return new Color(255, 196, 37);
            case PINK:
                return new Color(255,0,101);
            case BROWN:
                return new Color(224,168,153);
            case TEAL:
                return new Color(102,178,178);
            default:
                return null;
        }
    }

    /**
     * Initializes the GUI.
     *
     * @param width     The width of the board
     * @param height    The height of the board
     */
    private void initialize(int width, int height) {
        JFrame frame = new JFrame("Four in a row");
        frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        int fullWidth = BUTTON_WIDTH * width + (WIDTH_GAP + 1) * width;
        int fullHeight = BUTTON_HEIGHT + (TILE_HEIGHT * height) + HEIGHT_GAP * (height) + TILE_HEIGHT;

        this.frame = frame;
        JPanel container = new JPanel();
        container.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        container.setPreferredSize(new Dimension(fullWidth, fullHeight + TILE_HEIGHT + 2 * BUTTON_HEIGHT));
        frame.add(container);

        JPanel panel3 = new JPanel();
        this.resetButton = new JButton("Reset");
        this.resetButton.addActionListener(this);
        panel3.add(this.resetButton);
        for (IPlayer player : board.getPlayers()) {
            JLabel playerLabel = new JLabel(player.getName());
            playerLabel.setForeground(player.getColor());
            panel3.add(playerLabel);
        }
        container.add(panel3);

        JPanel panel2 = new JPanel();
        this.mainPanel = panel2;
        panel2.setBackground(new Color(0, 0, 200));
        panel2.setLayout(new FlowLayout(FlowLayout.CENTER, HEIGHT_GAP, WIDTH_GAP));
        panel2.setPreferredSize(new Dimension(fullWidth, fullHeight));
        dropButtons = new JButton[width];
        initializeDropButtons(width, panel2);
        tiles = new JTextPane[width * height];
        readTiles();
        container.add(panel2);

        infoLabel = new JLabel();
        infoLabel.setPreferredSize(new Dimension(fullWidth, BUTTON_HEIGHT));
        infoLabel.setBorder(new EmptyBorder(10, 10, 10, 10));
        container.add(infoLabel);

        eventLabel = new JLabel();
        eventLabel.setPreferredSize(new Dimension(fullWidth, BUTTON_HEIGHT));
        eventLabel.setBorder(new EmptyBorder(10, 10, 10, 10));
        container.add(eventLabel);

        frame.validate();
        frame.pack();
        frame.setVisible(true);
        currentPlayer = board.nextPlayer();
        displayMessage("It's " + currentPlayer.getName() + "'s turn.");
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        reset(e);
        if (!(board.won() || board.isFull())) {
            for (int i = 0; i < dropButtons.length; i++) {
                if (e.getSource() == dropButtons[i]) {
                    if (dropObject(i)) {
                        displayBoard();
                        if (board.won()) {
                            IGameObject winnerObject = board.get(i, ((IVerticalBoard)board).getTopObject(i, IPlayerObject.class));
                            displayMessage(((IPlayerObject)winnerObject).getOwner().getName() + " won.");
                        } else {
                            if (!board.isFull()) {
                                triggerEvents();
                            }
                            //This might look weird, but an event can fill up the board.
                            if (board.isFull()) {
                                displayMessage("It's a draw");
                            } else {
                                currentPlayer = board.nextPlayer();
                                displayMessage("It's " + currentPlayer.getName() + "'s turn.");
                                if (currentPlayer instanceof AI) {
                                    dropButtons[((AI)currentPlayer).getColumn((VerticalBoard)board)].doClick();
                                }
                            }
                        }
                    } else {
                        displayMessage("That column is full.");
                    }
                }
            }
        }
    }

    /**
     * Triggers a set of events after each player's turn.
     */
    private void triggerEvents() {
        if (bomb.rollDie(((IVerticalBoard)board), 2)) {
            eventLabel.setText(bomb.announcement());
        } else if (block.rollDie(((IVerticalBoard)board), 1)) {
            eventLabel.setText(block.announcement());
        } else {
            eventLabel.setText("");
        }
        while (((IVerticalBoard)board).floatingObjects() > 0) {
            ((IVerticalBoard)board).gravitate();
        }
        displayBoard();
    }
}
