package inf101.v18.connectfour.interfaces;

import inf101.v18.connectfour.board.IVerticalBoard;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.Token;

import javax.swing.*;
import java.awt.*;

/**
 * Implements some methods specific for connect four.
 */
public abstract class ConnectGUI extends GUI implements IVerticalGameInterface {
    JButton[] dropButtons;
    JTextPane[] tiles;

    @Override
    public void displayBoard() {
        for (JTextPane pane : this.tiles) {
            this.mainPanel.remove(pane);
        }
        this.tiles = new JTextPane[this.board.getWidth() * this.board.getHeight()];
        readTiles();
        this.frame.validate();
    }

    /**
     * Reads the values from board, and creates corresponding text panes.
     */
    void readTiles() {
        int k = 0;
        for (int i = board.getHeight() - 1; i > -1; i--) {
            for (int j = 0; j < board.getWidth(); j++) {
                JTextPane textPane = new JTextPane();
                this.tiles[k++] = textPane;
                textPane.setEditable(false);
                textPane.setBackground(new Color(0, 0, 200));
                IGameObject obj = board.get(j,i);
                textPane.setBorder(BorderFactory.createLineBorder(obj.getColor(), 50, true));
                textPane.setPreferredSize(new Dimension(BUTTON_WIDTH, TILE_HEIGHT));
                mainPanel.add(textPane, BorderLayout.PAGE_START);
            }
        }
    }

    /**
     * Creates a bunch of buttons.
     *
     * @param width     The width of the game grid
     * @param panel2    The panel the buttons should be attached to
     */
    void initializeDropButtons(int width, JPanel panel2) {
        for (int i = 0; i < width; i++) {
            JButton button = new JButton();
            button.addActionListener(this);
            button.setPreferredSize(new Dimension(BUTTON_WIDTH, BUTTON_HEIGHT));
            dropButtons[i] = button;
            panel2.add(button, BorderLayout.PAGE_START);
        }
    }

    @Override
    public boolean dropObject(int i) {
        return ((IVerticalBoard)board).drop(i, new Token(currentPlayer));
    }
}
