package inf101.v18.connectfour.interfaces;

import inf101.v18.connectfour.board.Board;
import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.events.winning.ThreeInARow;
import inf101.v18.connectfour.objects.Empty;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.Token;
import inf101.v18.connectfour.player.Player;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

/**
 * A Tic-Tac-Toe implementation using a Graphical User Interface.
 */
public class TicTacToeGUI extends GUI {
    private JButton[] gameGrid = new JButton[9];

    public TicTacToeGUI() {
        List<IWinCondition> conditions = new ArrayList<>();
        conditions.add(new ThreeInARow());
        this.board = new Board(3,3, new Empty(), conditions);
        int playerNum = 2;
        for (int i = 0; i < playerNum; i++) {
            String playerName = "";
            while (playerName.equals("")) {
                playerName = JOptionPane.showInputDialog(String.format("Name of player %d: ", i+1));
                if (playerName == null) {
                    System.exit(0);
                }
                if (playerName.length() > 10) {
                    JOptionPane.showMessageDialog(null, "Player name can't be longer than 10 characters.", "Error", JOptionPane.ERROR_MESSAGE);
                    playerName = "";
                }
            }
            Player player;
            if (i == 1) {
                player = new Player(playerName, Color.BLACK);
            } else {
                player = new Player(playerName, Color.WHITE);
            }
            if (!this.board.addPlayer(player)) {
                JOptionPane.showMessageDialog(null, "Player is not unique. Please try again.", "Error", JOptionPane.ERROR_MESSAGE);
                i--;
            }
        }
        initialize();
    }

    /**
     * Initializes the GUI.
     */
    private void initialize() {
        JFrame frame = new JFrame("Four in a row");
        frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        int fullWidth = (BUTTON_WIDTH) * 3;
        int fullHeight = (TILE_HEIGHT) * 3;

        this.frame = frame;
        JPanel container = new JPanel();
        container.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
        container.setPreferredSize(new Dimension(fullWidth, fullHeight + 2 * BUTTON_HEIGHT));
        frame.add(container);

        JPanel panel2 = new JPanel();
        mainPanel = panel2;
        panel2.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
        panel2.setPreferredSize(new Dimension(fullWidth, fullHeight));
        readTiles();
        container.add(panel2);

        infoLabel = new JLabel();
        infoLabel.setPreferredSize(new Dimension(fullWidth, 15));
        infoLabel.setBorder(new EmptyBorder(10, 10, 15, 10));
        container.add(infoLabel);
        resetButton = new JButton("Reset");
        resetButton.addActionListener(this);
        container.add(resetButton);

        frame.validate();
        frame.pack();
        frame.setVisible(true);
        currentPlayer = board.nextPlayer();
        displayMessage("It's " + currentPlayer.getName() + "'s turn.");
    }

    @Override
    public void displayBoard() {
        for (JButton button : gameGrid) {
            mainPanel.remove(button);
        }
        gameGrid = new JButton[board.getWidth() * board.getHeight()];
        readTiles();
        frame.validate();
    }

    @Override
    public boolean hasWon() {
        return board.won();
    }

    @Override
    public boolean placePlayerObject(int x, int y) {
        if (!board.isFull()) {
            try {
                board.set(x, y, new Token(currentPlayer));
                return true;
            } catch (IllegalArgumentException ignored) {}
        }
        return false;
    }

    /**
     * Converts the data contained in the board to buttons.
     */
    private void readTiles() {
        int k = 0;
        for (int i = 0; i < board.getHeight(); i++) {
            for (int j = 0; j < board.getWidth(); j++) {
                JButton button = new JButton();
                button.addActionListener(this);
                IGameObject obj = board.get(j,i);
                if (obj.getColor() == Color.black) { //This is a bit hacky, but prevents unnecessary changes,
                    button.setText("X");
                } else if (obj.getColor() == Color.white) {
                    button.setText("O");
                }
                button.setPreferredSize(new Dimension(BUTTON_WIDTH, TILE_HEIGHT));
                gameGrid[k++] = button;
                mainPanel.add(button, BorderLayout.PAGE_START);
            }
        }
    }

    /**
     * Handles all player input and output.
     *
     * @param e An event handler
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == resetButton) {
            this.board.flush();
            displayBoard();
            this.currentPlayer = board.nextPlayer();
            displayMessage("It's " + currentPlayer.getName() + "'s turn.");
        }
        if (!(board.won() || board.isFull())) {
            for (int i = 0; i < gameGrid.length; i++) {
                if (e.getSource() == gameGrid[i] && gameGrid[i].getText().equals("")) {
                    doTurn(i);
                }
            }
        }
    }

    /**
     * Places a token, and prepares the board and itself for the next event.
     *
     * @param i The index of the button pushed
     */
    private void doTurn(int i) {
        int x = i % 3;
        int y = i / 3;
        board.set(x, y, new Token(currentPlayer));
        displayBoard();
        if (board.won()) {
            IGameObject winnerObject = board.get(x, y);
            displayMessage(((Token)winnerObject).getOwner().getName() + " won.");
        } else {
            currentPlayer = board.nextPlayer();
            displayMessage("It's " + currentPlayer.getName() + "'s turn.");
            if (board.isFull()) {
                displayMessage("It's a draw");
            }
        }
    }
}
