package inf101.v18.connectfour;

import inf101.v18.connectfour.board.*;
import inf101.v18.connectfour.objects.*;
import inf101.v18.connectfour.player.Player;
import org.junit.Test;

import java.awt.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

public class GridTest {
    @Test(expected = IllegalArgumentException.class)
    public void negativeWidthTest() {
        new Grid<>(-1, 1, new Empty());
    }

    @Test(expected = IllegalArgumentException.class)
    public void negativeHeightTest() {
        new Grid<>(1, -1, new Empty());
    }

    @Test(expected = IllegalArgumentException.class)
    public void zeroWidthTest() {
        new Grid<>(0, 1, new Empty());
    }

    @Test(expected = IllegalArgumentException.class)
    public void zeroHeightTest() { new Grid<>(1, 0, new Empty()); }

    @Test
    public void gridWidthTest() {
        IGrid<IGameObject> grid = new Grid<>(10, 11, new Empty());
        assertEquals(10, grid.getWidth());
        grid = new Grid<>(3, 11, new Empty());
        assertEquals(3, grid.getWidth());
    }

    @Test
    public void gridHeightTest() {
        IGrid<IGameObject> grid = new Grid<>(10, 11, new Empty());
        assertEquals(11, grid.getHeight());
        grid = new Grid<>(10, 4, new Empty());
        assertEquals(4, grid.getHeight());
    }

    @Test
    public void gridSetGetTest() {
        IGrid<IGameObject> grid = new Grid<>(10, 11, new Empty());
        IGameObject imm = new Immovable();
        assertNotEquals(imm, grid.get(2, 2));
        grid.set(2, 2, imm);
        assertEquals(imm, grid.get(2, 2));
        assertNotEquals(imm, grid.get(3, 2));
    }

    @Test
    public void copyTest() {
        IGrid<IGameObject> grid = new Grid<>(5, 5, new Empty());
        IPlayerObject token = new Token(new Player("Naem", Color.pink));
        grid.set(0, 0, token);
        IGrid<IGameObject> gridCopy = grid.copy();
        assertEquals(token, gridCopy.get(0, 0));
        gridCopy.set(0, 3, token);

        assertEquals(token, gridCopy.get(0, 3));
        assertNotEquals(token, grid.get(0, 3));
    }

    @Test
    public void flushTest() {
        IGameObject empty = new Empty();
        IPlayerObject token = new Token(new Player("AASD", Color.MAGENTA));
        IGrid<IGameObject> grid = new Grid<>(5, 5, empty);
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                assertEquals(empty, grid.get(i, j));
                grid.set(i, j, token);
                assertEquals(token, grid.get(i, j));
            }
        }
        grid.flush();
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                assertEquals(empty, grid.get(i, j));
            }
        }
    }
}
