package inf101.v18.connectfour;

import inf101.v18.connectfour.board.IVerticalBoard;
import inf101.v18.connectfour.board.VerticalBoard;
import inf101.v18.connectfour.objects.*;
import inf101.v18.connectfour.player.Player;
import org.junit.Assert;
import org.junit.Test;

import java.awt.*;
import java.util.ArrayList;

import static org.junit.Assert.*;

public class VerticalBoardTest {
    @Test
    public void topObjectTest() {
        IVerticalBoard board = new VerticalBoard(10, 11, new Empty(), new ArrayList<>());
        assertEquals(-1, board.getTopObject(0));
        assertEquals(-1, board.getTopObject(0, Token.class));
        assertEquals(-1, board.getTopObject(0, Immovable.class));
        assertEquals(10, board.getTopObject(0, Empty.class));
        board.drop(0, new Token(new Player("Hello", Color.pink)));
        assertEquals(0, board.getTopObject(0));
        assertEquals(0, board.getTopObject(0, Token.class));
        assertEquals(-1, board.getTopObject(0, Immovable.class));
        assertEquals(10, board.getTopObject(0, Empty.class));
        board.drop(1, new Immovable());
        assertEquals(0, board.getTopObject(1));
        assertEquals(-1, board.getTopObject(1, Token.class));
        assertEquals(0, board.getTopObject(1, Immovable.class));
        assertEquals(10, board.getTopObject(1, Empty.class));
    }

    @Test
    public void dropTest1() {
        IVerticalBoard board = new VerticalBoard(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        Assert.assertNotSame(board.get(0, 0), token);
        board.drop(0, token);
        assertEquals(token, board.get(0, 0));
    }

    @Test
    public void dropTest2() {
        IVerticalBoard board = new VerticalBoard(10, 11, new Empty(), new ArrayList<>());
        IGameObject imm = new Immovable();
        assertNotEquals(imm, board.get(5, 0));
        board.drop(5, imm);
        assertEquals(imm, board.get(5, 0));
        assertEquals(imm, board.get(5, board.getTopObject(5)));
    }

    @Test
    public void gravitateTest() {
        IVerticalBoard board = new VerticalBoard(4, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("ÆÆÆ", Color.RED));
        board.set(0, 4, token);
        board.gravitate();
        assertEquals(token, board.get(0, 3));
        board.gravitate();
        assertEquals(token, board.get(0, 2));
        board.gravitate();
        assertEquals(token, board.get(0, 1));
        board.gravitate();
        assertEquals(token, board.get(0, 0));
    }

    @Test
    public void floatingObjectsEasyTest() {
        IVerticalBoard board = new VerticalBoard(1, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("ÆÆÆ", Color.RED));
        board.set(0, 4, token);
        assertEquals(4, board.floatingObjects());
        board.gravitate();
        assertEquals(3, board.floatingObjects());
        board.gravitate();
        assertEquals(2, board.floatingObjects());
        board.gravitate();
        assertEquals(1, board.floatingObjects());
        board.gravitate();
        assertEquals(0, board.floatingObjects());
    }

    @Test
    public void floatingObjectsComplicatedTest() {
        VerticalBoard board = new VerticalBoard(4, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("ÆÆÆ", Color.RED));
        assertEquals(0, board.floatingObjects());
        board.set(0, 4, token);
        assertEquals(4, board.floatingObjects());
        board.set(0, 2, token);
        assertEquals(3, board.floatingObjects());
        board.set(1, 3, token);
        assertEquals(3, board.floatingObjects());
        board.set(3, 4, token);
        assertEquals(4, board.floatingObjects());
        board.set(3, 3, token);
        assertEquals(3, board.floatingObjects());
    }

    @Test(expected = IllegalArgumentException.class)
    public void dropNegativeColumnTest() {
        IVerticalBoard board = new VerticalBoard(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        board.drop(-1, token);
    }

    @Test(expected = IllegalArgumentException.class)
    public void dropOutOfBoundsTest() {
        IVerticalBoard board = new VerticalBoard(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        board.drop(5, token);
    }

    @Test
    public void isFullTest() {
        IVerticalBoard board = new VerticalBoard(5, 5, new Empty(), new ArrayList<>());
        IGameObject token = new Token(new Player("Naem", Color.green));
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.drop(0, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.drop(1, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.drop(2, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.drop(3, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.drop(4, token);
        }
        assertTrue(board.isFull());
    }
}
